#!/usr/bin/env python

# Author                 Version      Date
# -----------------------------------------------
# Martin Still (SAAO)    0.2          31 Jul 2007
# S M Crawford (SAAO)    0.3          22 Feb 2008

# saltclean performs the standard chain of primary reduction tasks on 
# lists of RSS or SALTICAM images.

import saltprint, saltio
import sys,glob, os, shutil
from sets import Set
from pyraf import iraf 
from pyraf.iraf import pysalt
from pyraf.iraf import saltred

# -----------------------------------------------------------
# core routine

def saltclean(rawpath,outpath,obslog,gaindb,xtalkfile,geomfile,subover,trim,median,function,
	      order,rej_lo,rej_hi,niter,masbias,subbias,interp,cleanup,clobber,logfile,verbose,
	      status):


# test the logfile

    workpath, status = saltio.abspath('.',logfile)
    if (logfile[0] != '/'): logfile = workpath+logfile
    logfile = saltio.logname(logfile)

# log the call 

    saltprint.line(logfile,verbose)
    message = 'SALTCLEAN -- '
    message += 'rawpath='+rawpath+' '
    message += 'outpath='+outpath+' '
    message += 'obslog='+obslog+' '
    message += 'gaindb='+gaindb+' '
    message += 'xtalkfile='+xtalkfile+' '
    message += 'geomfile='+geomfile+' '
    yn = 'n'
    if (subover): yn = 'y'
    message += 'subover='+yn+' '
    yn = 'n'
    if (trim): yn = 'y'
    message += 'trim='+yn+' '
    yn = 'n'
    if (median): yn = 'y'
    message += 'median='+yn+' '
    message += 'function='+function+' '
    message += 'order='+str(order)+' '
    message += 'rej_lo='+str(rej_lo)+' '
    message += 'rej_hi='+str(rej_hi)+' '
    message += 'niter='+str(niter)+' '
    yn = 'n'
    if (masbias): yn = 'y'
    message += 'masbias='+yn+' '
    yn = 'n'
    if (subbias): yn = 'y'
    message += 'subbias='+yn+' '
    yn = 'n'
    message += 'interp='+interp+' '
    if (cleanup): yn = 'y'
    message += 'cleanup='+yn+' '
    yn = 'n'
    if (clobber): yn = 'y'
    message += 'clobber='+yn+' '
    message += 'logfile='+logfile+' '
    yn = 'n'
    if (verbose): yn = 'y'
    message += 'verbose='+yn
    saltprint.log(logfile,message,verbose)

# start time

    saltprint.time('\nSALTCLEAN -- started at',logfile,verbose)
    saltprint.log(logfile,' ',verbose)

# check the rawpath exists and ends with a "/". convert to absolute path

    if (status == 0): rawpath, status = saltio.pathexists(rawpath,logfile)
    if (status == 0): rawpath, status = saltio.abspath(rawpath,logfile)

# check the outpath exists and ends with a "/". convert to absolute path

    if (status == 0): outpath, status = saltio.pathexists(outpath,logfile)
    if (status == 0): outpath, status = saltio.abspath(outpath,logfile)

# read observation log

    if (status == 0): obstruct,status = saltio.openfits(obslog,logfile)
    if (status == 0): obstab,status = saltio.readtab(obstruct[1],obslog,logfile)
    if (status == 0): status = saltio.closefits(obstruct,logfile)

# SALTPREPARE: prepare files for IRAF/PyRAF tasks

    if (status == 0): status = run_saltprepare(rawpath,outpath,obstab,clobber,logfile, verbose,status)

# SALTGAIN: gain correct image files

    if (status == 0): status = run_saltgain(outpath,obstab,gaindb,clobber,logfile,verbose,status)

# SALTXTALK: correct images for cross talk ghosts

    if (status == 0): status = run_saltxtalk(outpath,obstab,xtalkfile,clobber,logfile,verbose,status)

# SALTBIAS: create master bias frames

    if (status == 0): status = run_saltbias_master(outpath,obstab,subover,trim,median,function, \
						   order,rej_lo,rej_hi,niter,clobber,logfile,   \
						   verbose,status)

# SALTBIAS: remove bias from images

    if (status == 0): status = run_saltbias(outpath,obstab,subover,trim,median,function,order,rej_lo, \
					    rej_hi,niter,clobber,logfile,verbose,status)

# SALTMOSAIC: mosaic images amplifiers

    if (status == 0): status = run_saltmosaic(outpath,obstab,geomfile,interp,cleanup,clobber,logfile,
					      verbose,status)

# SALTSLOT: clean raw slot mode images

    if (status == 0): status = run_saltslot(rawpath,outpath,obstab,gaindb,xtalkfile,clobber,logfile,
					    verbose,status)

# delete intermediate files

    if (status == 0 and cleanup):
	filenames = obstab.field('filename')
	detmodes = obstab.field('detmode')
	message = '\nSALTCLEAN -- deleting intermediate files p*.fits xp*.fits and bxp*.fits'
	saltprint.log(logfile,message,verbose)
	for i in range(len(filenames)):
	    if (detmodes[i] != 'Slot Mode' and detmodes[i] != 'Video Mode'):
		status = saltio.delete(outpath+'p'+filenames[i],False,logfile)
		status = saltio.delete(outpath+'xp'+filenames[i],False,logfile)
		status = saltio.delete(outpath+'bxp'+filenames[i],False,logfile)

# end time

    if (status == 0):
	saltprint.time('\nSALTCLEAN -- completed at',logfile,verbose)
    else:
	saltprint.time('\nSALTCLEAN -- aborted at',logfile,verbose)

# -----------------------------------------------------------
# saltprepare task

def run_saltprepare(rawpath,outpath,obstab,clobber,logfile,verbose,status):

    images = '' # list of files to prepared

# read observation log

    filenames = obstab.field('filename')
    detmodes = obstab.field('detmode')
    instrumes = obstab.field('instrume')

# determine instrument

    message = 'ERROR -- SALTCLEAN: cannot determine instrument type from the first exposure'
    if (instrumes[0] != 'SALTICAM' and instrumes[0] != 'RSS' and instrumes[0] != 'PFIS'):
	status = saltprint.err(logfile,message)

# filter files to remove slot mode data

    message = 'ERROR -- SALTCLEAN: observation log contains images from multiple instruments'
    for i in range(len(filenames)):
	if (detmodes[i] != 'Video Mode' and detmodes[i] != 'Slot Mode'):
	    images += filenames[i] + ','

# check all files originated from one instrument only

	if (instrumes[i] != instrumes[0]):
	    status = saltprint.err(logfile,verbose)
    images = images.rstrip(',')

# execute the task

    if (status == 0 and len(images) > 0): 
	saltred.saltprepare(images=images,rawpath=rawpath,outimages='',outpref=outpath+'p',  \
				   clobber=clobber,logfile=logfile,verbose=verbose,status=status)

    return status

# -----------------------------------------------------------
# saltgain task

def run_saltgain(outpath,obstab,gaindb,clobber,logfile,verbose,status):

    gainfiles = '' # list of files to be corrected

# read observatgion log

    filenames = obstab.field('filename')
    detmodes = obstab.field('detmode')
    instrumes = obstab.field('instrume')

# filter files to remove slot mode data

    if (status == 0):
	for i in range(len(filenames)):
	    if (detmodes[i] != 'Video Mode' and detmodes[i] != 'Slot Mode'):
		gainfiles += outpath + 'p' + filenames[i] + ','
	gainfiles = gainfiles.rstrip(',')

# execute the task

    if (status == 0 and len(gainfiles) > 0): 
	saltred.saltgain(images=gainfiles,gaindb=gaindb,mult=True,
		      logfile=logfile,verbose=verbose,status=status)

    return status

# -----------------------------------------------------------
# saltxtalk task

def run_saltxtalk(outpath,obstab,xtalkfile,clobber,logfile,verbose,status):

    xtalkfiles = '' # list of files to be corrected

# read observation log

    filenames = obstab.field('filename')
    detmodes = obstab.field('detmode')
    instrumes = obstab.field('instrume')

# filter files to remove slot mode data

    if (status == 0):
	for i in range(len(filenames)):
	    if (detmodes[i] != 'Video Mode' and detmodes[i] != 'Slot Mode'):
		xtalkfiles += outpath + 'p' + filenames[i] + ','
	xtalkfiles = xtalkfiles.rstrip(',')

# execute the task

    if (status == 0 and len(xtalkfiles) > 0): 
	saltred.saltxtalk(images=xtalkfiles,outimages='',outpref=outpath+'x',
		       xtalkfile=xtalkfile,clobber=clobber,logfile=logfile,
		       verbose=verbose,status=status)

    return status

# -----------------------------------------------------------
# saltbias task: create master bias file(s)

def run_saltbias_master(outpath,obstab,subover,trim,median,function,order,rej_lo,rej_hi,niter,
			clobber,logfile,verbose,status):

    biasfiles = ''

# read observation log

    filenames = obstab.field('filename')
    detmodes = obstab.field('detmode')
    ccdsums = obstab.field('ccdsum')
    gainsets = obstab.field('gainset')
    rospeeds = obstab.field('rospeed')
    ccdtypes = obstab.field('ccdtype')

# construct master bias file name

    if (status == 0):
	for bin in Set(ccdsums):
	    bnam = bin.replace(' ','x')
	    for gain in Set(gainsets):
		if (gain == 'BRIGHT'): 
		    gnam = 'Br'
		elif (gain == 'FAINT'):
		    gnam = 'Fa'
		else:
		    gnam = 'Un'
		for speed in Set(rospeeds):
		    if (speed == 'FAST'): 
			snam = 'Fa'
		    elif (speed == 'SLOW'):
			snam = 'Sl'
		    else:
			snam = 'Un'
		    for mode in Set(detmodes):
			if (mode == 'Normal'): 
			    mnam = ''
			elif (mode == 'Frame Transfer'):
			    mnam = 'FT'
			elif (mode == 'Video Mode'):
			    mnam = 'Sl'
			else:
			    mnam = 'Un'

# filter images for binning, gain, readout speed and instrument mode

			biasfiles = ''
			for i in range(len(filenames)):
			    if (bin == ccdsums[i] and gain == gainsets[i] and \
				detmodes[i] != 'Slot Mode' and detmodes[i] != 'Video Mode' and \
				speed == rospeeds[i] and mode == detmodes[i] and \
				(ccdtypes[i] == 'ZERO' or ccdtypes[i] == 'BIAS')):
				biasfiles += ',' + outpath + 'xp' + filenames[i]
				masterbias  = outpath + filenames[i][:9] + 'Bias'
				masterbias += mnam + bnam + snam + gnam + '.fits'
			biasfiles = biasfiles.lstrip(',')

# execute the task

			if (status == 0 and len(biasfiles) > 0): 
			    saltred.saltbias(images=biasfiles,outimages='',outpref=outpath+'b',
					  subover=subover,trim=trim,subbias=False,crbias=True,
					  masterbias=masterbias,median=median,function=function,
					  order=order,rej_lo=rej_lo,rej_hi=rej_hi,niter=niter,
					  plotover=False,turbo=False,logfile=logfile,clobber=clobber,
					  verbose=verbose,status=status)

    return status 

# -----------------------------------------------------------
# saltbias task: debias science images

def run_saltbias(outpath,obstab,subover,trim,median,function,order,rej_lo,rej_hi,niter,
		 clobber,logfile,verbose,status):

    biasfiles = ''

# read observation log

    filenames = obstab.field('filename')
    detmodes = obstab.field('detmode')
    ccdsums = obstab.field('ccdsum')
    gainsets = obstab.field('gainset')
    rospeeds = obstab.field('rospeed')
    ccdtypes = obstab.field('ccdtype')

# construct master bias file name

    if (status == 0):
	for bin in Set(ccdsums):
	    bnam = bin.replace(' ','x')
	    for gain in Set(gainsets):
		if (gain == 'BRIGHT'): 
		    gnam = 'Br'
		elif (gain == 'FAINT'):
		    gnam = 'Fa'
		else:
		    gnam = 'Un'
		for speed in Set(rospeeds):
		    if (speed == 'FAST'): 
			snam = 'Fa'
		    elif (speed == 'SLOW'):
			snam = 'Sl'
		    else:
			snam = 'Un'
		    for mode in Set(detmodes):
			if (mode == 'Normal'): 
			    mnam = ''
			elif (mode == 'Frame Transfer'):
			    mnam = 'FT'
			elif (mode == 'Video Mode'):
			    mnam = 'Sl'
			else:
			    mnam = 'Un'
			masterbias = ''
			subbias = False
			for masterbias in glob.glob(outpath + '*Bias' + mnam + bnam + snam + gnam + '.fits'):
#			    subbias = True
                            subbias = False

# filter images for binning, gain, readout speed and instrument mode

			biasfiles = ''
			for i in range(len(filenames)):
			    if (bin == ccdsums[i] and gain == gainsets[i] and \
				speed == rospeeds[i] and detmodes[i] != 'Video Mode' and \
				detmodes[i] != 'Slot Mode' and mode == detmodes[i] and \
				ccdtypes[i] != 'ZERO' and ccdtypes[i] != 'BIAS'):
				biasfiles += ',' + outpath + 'xp' + filenames[i]
			biasfiles = biasfiles.lstrip(',')

# execute the task

			if (status == 0 and len(biasfiles) > 0): 
			    saltred.saltbias(images=biasfiles,outimages='',outpref=outpath+'b',
					  subover=subover,trim=trim,subbias=subbias,
					  crbias=False,masterbias=masterbias,median=median,
					  function=function,order=order,rej_lo=rej_lo,
					  rej_hi=rej_hi,niter=niter,plotover=False,turbo=False,
					  logfile=logfile,clobber=clobber,verbose=verbose,
					  status=status)

    return status 

# -----------------------------------------------------------
# saltmosaic task: mosaic amplifier images into single image

def run_saltmosaic(outpath,obstab,geomfile,interp,cleanup,clobber,logfile,verbose,status):

    mosaicfiles = '' # list of files to be mosaiced

# read observation log

    filenames = obstab.field('filename')
    detmodes = obstab.field('detmode')

# filter files to remove slot mode data

    if (status == 0):
	for i in range(len(filenames)):
	    if (detmodes[i] != 'Video Mode' and detmodes[i] != 'Slot Mode'):
		mosaicfiles += outpath + 'bxp' + filenames[i] + ','
	mosaicfiles = mosaicfiles.rstrip(',')

# execute the task

    if (status == 0 and len(mosaicfiles) > 0): 
	saltred.saltmosaic(images=mosaicfiles,
			outimages='',outpref=outpath+'m',geomfile=geomfile,
			interp=interp,cleanup=cleanup,clobber=clobber,logfile=logfile,
			verbose=verbose,status=status)

    return status

# -----------------------------------------------------------
# saltslot task: reduce slot mode data from raw

def run_saltslot(rawpath,outpath,obstab,gaindb,xtalkfile,clobber,logfile,verbose,status):

    slotfiles = ''

# read observation log

    filenames = obstab.field('filename')
    detmodes = obstab.field('detmode')
    instrumes = obstab.field('instrume')

# filter slot mode from image list

    if (status == 0):
	for i in range(len(filenames)):
	    if (detmodes[i] == 'Video Mode' or detmodes[i] == 'Slot Mode'):
		slotfiles += ',' + rawpath + filenames[i]
	slotfiles = slotfiles.lstrip(',')

# execute task

    if (status == 0 and len(slotfiles) > 0): 
	saltred.saltslot(images=slotfiles,outimages='',outpref=outpath+'bxp',gaindb=gaindb,
		      xtalkfile=xtalkfile,clobber=clobber,logfile=logfile,verbose=verbose,
		      status=status)

    return status

# -----------------------------------------------------------
# main code 

parfile = iraf.osfn("saltred$saltclean.par") 
t = iraf.IrafTaskFactory(taskname="saltclean",value=parfile,function=saltclean, pkgname='saltred')
